@@ -200,12 +200,13 @@ class Job(AbstractJob):
         self.rule.expand_benchmark(self.wildcards_dict)
         self.rule.expand_log(self.wildcards_dict)
 
-    def outputs_older_than_script(self):
+    def outputs_older_than_script_or_notebook(self):
         """return output that's older than script, i.e. script has changed"""
-        if not self.is_script:
+        path = self.rule.script or self.rule.notebook
+        if not path:
             return
-        assert os.path.exists(self.rule.script)  # to make sure lstat works
-        script_mtime = os.lstat(self.rule.script).st_mtime
+        assert os.path.exists(path)  # to make sure lstat works
+        script_mtime = os.lstat(path).st_mtime
         for f in self.expanded_output:
             if f.exists:
                 if not f.is_newer(script_mtime):
@@ -391,6 +392,10 @@ class Job(AbstractJob):
     def is_script(self):
         return self.rule.script is not None
 
+    @property
+    def is_notebook(self):
+        return self.rule.notebook is not None
+
     @property
     def is_wrapper(self):
         return self.rule.wrapper is not None
@@ -1045,11 +1050,13 @@ class GroupJob(AbstractJob):
         "_inputsize",
         "_all_products",
         "_attempt",
+        "toposorted",
     ]
 
     def __init__(self, id, jobs):
         self.groupid = id
         self.jobs = frozenset(jobs)
+        self.toposorted = None
         self._resources = None
         self._input = None
         self._output = None
@@ -1067,15 +1074,15 @@ class GroupJob(AbstractJob):
         self.jobs = self.jobs | other.jobs
 
     def finalize(self):
-        # TODO determine resources based on toposort
-        dag = {
-            job: {dep for dep in self.dag.dependencies[job] if dep in self.jobs}
-            for job in self.jobs
-        }
         from toposort import toposort
 
-        t = toposort(dag)
-        print(t)
+        if self.toposorted is None:
+            dag = {
+                job: {dep for dep in self.dag.dependencies[job] if dep in self.jobs}
+                for job in self.jobs
+            }
+
+            self.toposorted = list(toposort(dag))
 
     @property
     def all_products(self):
@@ -1084,7 +1091,10 @@ class GroupJob(AbstractJob):
         return self._all_products
 
     def __iter__(self):
-        return iter(self.jobs)
+        if self.toposorted is None:
+            yield from self.jobs
+        else:
+            yield from chain.from_iterable(self.toposorted)
 
     def __repr__(self):
         return "JobGroup({},{})".format(self.groupid, repr(self.jobs))
@@ -1157,25 +1167,34 @@ class GroupJob(AbstractJob):
             pipe_group = any(
                 [any([is_flagged(o, "pipe") for o in job.output]) for job in self.jobs]
             )
-            for job in self.jobs:
-                try:
-                    job_resources = job.resources
-                except FileNotFoundError:
-                    # Skip job if resource evaluation leads to a file not found error.
-                    # This will be caused by an inner job, which needs files created by the same group.
-                    # All we can do is to ignore such jobs for now.
-                    continue
-                for res, value in job_resources.items():
-                    if self.dag.workflow.run_local or pipe_group:
-                        # in case of local execution, this must be a
-                        # group of jobs that are connected with pipes
-                        # and have to run simultaneously
-                        self._resources[res] += value
-                    else:
-                        # take the maximum over all jobs
-                        self._resources[res] = max(
-                            self._resources.get(res, value), value
-                        )
+            # iterate over siblings that can be executed in parallel
+            for siblings in self.toposorted:
+                sibling_resources = defaultdict(int)
+                for job in siblings:
+                    try:
+                        job_resources = job.resources
+                    except FileNotFoundError:
+                        # Skip job if resource evaluation leads to a file not found error.
+                        # This will be caused by an inner job, which needs files created by the same group.
+                        # All we can do is to ignore such jobs for now.
+                        continue
+                    for res, value in job_resources.items():
+                        if res != "_nodes":
+                            sibling_resources[res] += value
+
+                for res, value in sibling_resources.items():
+                    if res != "_nodes":
+                        if self.dag.workflow.run_local or pipe_group:
+                            # in case of local execution, this must be a
+                            # group of jobs that are connected with pipes
+                            # and have to run simultaneously
+                            self._resources[res] += value
+                        else:
+                            # take the maximum with previous values
+                            self._resources[res] = max(
+                                self._resources.get(res, 0), value
+                            )
+
         return Resources(fromdict=self._resources)
 
     @property
