@@ -52,7 +52,9 @@ class RemoteProvider(AbstractRemoteProvider):
 
 
 class RemoteObject(AbstractRemoteObject):
-    def __init__(self, *args, keep_local=False, provider=None, **kwargs):
+    def __init__(
+        self, *args, keep_local=False, provider=None, user_project=None, **kwargs
+    ):
         super(RemoteObject, self).__init__(
             *args, keep_local=keep_local, provider=provider, **kwargs
         )
@@ -62,6 +64,9 @@ class RemoteObject(AbstractRemoteObject):
         else:
             self.client = storage.Client(*args, **kwargs)
 
+        # keep user_project available for when bucket is initialized
+        self._user_project = user_project
+
         self._key = None
         self._bucket_name = None
         self._bucket = None
@@ -127,7 +132,7 @@ class RemoteObject(AbstractRemoteObject):
 
     @lazy_property
     def bucket(self):
-        return self.client.bucket(self.bucket_name)
+        return self.client.bucket(self.bucket_name, user_project=self._user_project)
 
     @lazy_property
     def blob(self):
