@@ -95,6 +95,7 @@ class Rule:
             self.run_func = None
             self.shellcmd = None
             self.script = None
+            self.notebook = None
             self.wrapper = None
             self.cwl = None
             self.norun = False
@@ -139,6 +140,7 @@ class Rule:
             self.run_func = other.run_func
             self.shellcmd = other.shellcmd
             self.script = other.script
+            self.notebook = other.notebook
             self.wrapper = other.wrapper
             self.cwl = other.cwl
             self.norun = other.norun
@@ -925,7 +927,9 @@ class Rule:
                     raise InputFunctionException(e, rule=self, wildcards=wildcards)
 
                 if not isinstance(res, int):
-                    raise WorkflowError("Resources function did not return int.")
+                    raise WorkflowError(
+                        "Resources function did not return int.", rule=self
+                    )
             res = min(self.workflow.global_resources.get(name, res), res)
             return res
 
