@@ -170,6 +170,7 @@ def listfiles(pattern, restriction=None, omit_value=None):
     else:
         dirname = os.path.dirname(pattern)
     pattern = re.compile(regex(pattern))
+
     for dirpath, dirnames, filenames in os.walk(dirname):
         for f in chain(filenames, dirnames):
             if dirpath != ".":
@@ -247,7 +248,7 @@ def report(
         metadata (str):     E.g. an optional author name or email address.
 
     """
-    if stylesheeet is None:
+    if stylesheet is None:
         os.path.join(os.path.dirname(__file__), "report.css")
     try:
         import snakemake.report
