@@ -55,6 +55,7 @@ from snakemake.io import (
 from snakemake.persistence import Persistence
 from snakemake.utils import update_config
 from snakemake.script import script
+from snakemake.notebook import notebook
 from snakemake.wrapper import wrapper
 from snakemake.cwl import cwl
 import snakemake.wrapper
@@ -76,6 +77,7 @@ class Workflow:
         overwrite_workdir=None,
         overwrite_configfiles=None,
         overwrite_clusterconfig=dict(),
+        overwrite_threads=dict(),
         config_args=None,
         debug=False,
         verbose=False,
@@ -129,6 +131,7 @@ class Workflow:
         self.overwrite_config = overwrite_config
         self.overwrite_configfiles = overwrite_configfiles
         self.overwrite_clusterconfig = overwrite_clusterconfig
+        self.overwrite_threads = overwrite_threads
         self.config_args = config_args
         self.immediate_submit = None
         self._onsuccess = lambda log: None
@@ -203,8 +206,9 @@ class Workflow:
         for f in self.included:
             files.add(os.path.relpath(f))
         for rule in self.rules:
-            if rule.script:
-                script_path = norm_rule_relpath(rule.script, rule)
+            script_path = rule.script or rule.notebook
+            if script_path:
+                script_path = norm_rule_relpath(script_path, rule)
                 files.add(script_path)
                 script_dir = os.path.dirname(script_path)
                 files.update(
@@ -438,6 +442,7 @@ class Workflow:
         report=None,
         export_cwl=False,
         batch=None,
+        keepincomplete=False,
     ):
 
         self.check_localrules()
@@ -709,7 +714,7 @@ class Workflow:
         elif list_code_changes:
             items = list(chain(*map(self.persistence.code_changed, dag.jobs)))
             for j in dag.jobs:
-                items.extend(list(j.outputs_older_than_script()))
+                items.extend(list(j.outputs_older_than_script_or_notebook()))
             if items:
                 print(*items, sep="\n")
             return True
@@ -787,6 +792,7 @@ class Workflow:
             greediness=greediness,
             force_use_threads=force_use_threads,
             assume_shared_fs=assume_shared_fs,
+            keepincomplete=keepincomplete,
         )
 
         if not dryrun:
@@ -1000,7 +1006,7 @@ class Workflow:
             # handle default resources
             if self.default_resources is not None:
                 rule.resources = copy.deepcopy(self.default_resources.parsed)
-            if ruleinfo.threads:
+            if ruleinfo.threads is not None:
                 if (
                     not isinstance(ruleinfo.threads, int)
                     and not isinstance(ruleinfo.threads, float)
@@ -1010,7 +1016,10 @@ class Workflow:
                         "Threads value has to be an integer, float, or a callable.",
                         rule=rule,
                     )
-                rule.resources["_cores"] = int(ruleinfo.threads)
+                if name in self.overwrite_threads:
+                    rule.resources["_cores"] = self.overwrite_threads[name]
+                else:
+                    rule.resources["_cores"] = int(ruleinfo.threads)
             if ruleinfo.shadow_depth:
                 if ruleinfo.shadow_depth not in (True, "shallow", "full", "minimal"):
                     raise RuleException(
@@ -1070,12 +1079,15 @@ class Workflow:
                 # a software stack specifically compiled for a particular
                 # HPC cluster.
                 invalid_rule = not (
-                    ruleinfo.script or ruleinfo.wrapper or ruleinfo.shellcmd
+                    ruleinfo.script
+                    or ruleinfo.wrapper
+                    or ruleinfo.shellcmd
+                    or ruleinfo.notebook
                 )
                 if invalid_rule:
                     raise RuleException(
                         "Modules directive is only allowed with "
-                        "shell, script or wrapper directives (not with run)",
+                        "shell, script, notebook, or wrapper directives (not with run)",
                         rule=rule,
                     )
                 from snakemake.deployment.env_modules import EnvModules
@@ -1083,10 +1095,15 @@ class Workflow:
                 rule.env_modules = EnvModules(*ruleinfo.env_modules)
             else:
                 if ruleinfo.conda_env and self.use_conda:
-                    if not (ruleinfo.script or ruleinfo.wrapper or ruleinfo.shellcmd):
+                    if not (
+                        ruleinfo.script
+                        or ruleinfo.wrapper
+                        or ruleinfo.shellcmd
+                        or ruleinfo.notebook
+                    ):
                         raise RuleException(
                             "Conda environments are only allowed "
-                            "with shell, script, or wrapper directives "
+                            "with shell, script, notebook, or wrapper directives "
                             "(not with run).",
                             rule=rule,
                         )
@@ -1101,13 +1118,16 @@ class Workflow:
 
                 if self.use_singularity:
                     invalid_rule = not (
-                        ruleinfo.script or ruleinfo.wrapper or ruleinfo.shellcmd
+                        ruleinfo.script
+                        or ruleinfo.wrapper
+                        or ruleinfo.shellcmd
+                        or ruleinfo.notebook
                     )
                     if ruleinfo.singularity_img:
                         if invalid_rule:
                             raise RuleException(
                                 "Singularity directive is only allowed "
-                                "with shell, script or wrapper directives "
+                                "with shell, script, notebook or wrapper directives "
                                 "(not with run).",
                                 rule=rule,
                             )
@@ -1122,6 +1142,7 @@ class Workflow:
             rule.run_func = ruleinfo.func
             rule.shellcmd = ruleinfo.shellcmd
             rule.script = ruleinfo.script
+            rule.notebook = ruleinfo.notebook
             rule.wrapper = ruleinfo.wrapper
             rule.cwl = ruleinfo.cwl
             rule.restart_times = self.restart_times
@@ -1275,6 +1296,13 @@ class Workflow:
 
         return decorate
 
+    def notebook(self, notebook):
+        def decorate(ruleinfo):
+            ruleinfo.notebook = notebook
+            return ruleinfo
+
+        return decorate
+
     def wrapper(self, wrapper):
         def decorate(ruleinfo):
             ruleinfo.wrapper = wrapper
@@ -1327,6 +1355,7 @@ class RuleInfo:
         self.docstring = None
         self.group = None
         self.script = None
+        self.notebook = None
         self.wrapper = None
         self.cwl = None
 
